/****************************************************************/
/*   This program download a waveform from a LECROY 9370	*/
/*   digital oscilloscope. Data points are saved in a Tcl	*/
/*   source file. Transfert is made through the GPIB interface. */
/*   Version 1.0 20/03/1997 (C) Jean-Marie Bussat		*/
/*--------------------------------------------------------------*/
/* Input parameters :						*/
/*	- Name of the channel to download			*/
/*	format : Ci (i=1,2)  i channel				*/
/*		 Tj (j=A,B,C,D) j mathematical function		*/
/*		 Mk (k=1,2,3,4) k memory			*/
/*	- Name of the file where datas are stored		*/
/****************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ib.h>
#include <math.h>
#include "lecroy9300.h"

#define DEBUG 1

#define DESC_HEAD 22

void main(argc,argv)
int argc;
char *argv[];
{
	int dso;		/* scope identifier		*/
	char cmd[20];		/* GPIB command			*/
	char *wave;		/* waveform buffer		*/
	descriptor *desc;	/* pointer on descriptor	*/
	char buffer[400];	/* descriptor buffer		*/
	char *filename;		/* name of the file !!!		*/
	FILE *data;		/* output file handle		*/
	long nbpts,i,j;		/* number of acquired points	*/
	float gain,offset;	/* vertical gain and offset	*/
	char bdt[16],cal[16];	/* timebase and calibre		*/
	short int rec_type;	/* acquisition type		*/
	float time_step;	/* time between 2 data points	*/
	float tdiv;		/* timebase : time/div		*/
	double	time_offset;	/* time offset / trigger.	*/
	float data_point_y;
	double data_point_x;
	long x_value;		/* for the output file		*/
	long save_size;		/* number of points to be saved	*/
	long save_inc;		/* saving increment		*/
	unsigned char temp;
	
	if(argc!=3) {
		fprintf(stderr,"********* ERROR - Bad number of parameters *********\n");
		fprintf(stderr,"Usage : %s channel filename\n",argv[0]);
		fprintf(stderr,"----------------------------------------------------\n");
		fprintf(stderr,"channel  : oscilloscope channel identifier\n");
		fprintf(stderr,"filename : name of the file where waveform is stored\n");
		exit(1);
	}

/* Hardware initialization					*/
	dso=gpib_find("LC9350"); /* get identifier		*/
	gpib_online(dso);	 /* GPIB interface online	*/
	gpib_SIC(dso);		 /* scope GPIB reset		*/
	gpib_REN(dso);		 /* Remote control		*/

/* Software initialization : exchange protocol definition	*/
/*   Finite length transmission (DEF9) on 8 bits (BYTE) and	*/
/*   binary datas (BIN)					*/
	gpib_write(dso,"CFMT DEF9,BYTE,BIN");
	gpib_write(dso,"CHRD SHORT"); /* minimal header 	*/
	gpib_write(dso,"CORD LO");    /* little endian format	*/
	
/* Descriptor request						*/
	sprintf(cmd,"%s:WF? DESC",argv[1]);
	gpib_write(dso,cmd);
	gpib_read(dso,buffer,370);
	
/* Remove descriptor header					*/
	desc=(descriptor *)(buffer+DESC_HEAD);

/* Find the most usefull waveforms parameters			*/

/*   Number of points : We add the two trailing zeros and	*/
/*   (2 points always sent by the oscilloscope) +		*/
/*   22 characters (header) (XX:WF DAT1,#90000nnnnn) +		*/
/*   2 bytes (why are they here ?)				*/		
	nbpts=desc->pnts_per_screen+26;

	offset=desc->vertical_offset;
	gain=desc->vertical_gain;
	rec_type=desc->record_type;
	time_step=desc->horiz_interval;
	tdiv=time_div[desc->timebase];
	
	if(tdiv==0.0) {
		fprintf(stderr,"ERROR - Unknown timebase index\n");
		exit(1);
	}
	
	if(tdiv==-1.0) {
		fprintf(stderr,"ERROR - External timebase not supported\n");
		exit(1);
	}
	
	time_offset=desc->horiz_offset;
	strcpy(bdt,timebase[desc->timebase]);
	strcpy(cal,fixed_vert_gain[desc->fixed_vert_gain]);

#ifdef DEBUG	
	printf("%d Number of points\n",nbpts);
	printf("Vertical gain          = %3.6e\n",gain);
	printf("Offset                 = %3.6e V\n",offset);
	printf("Time between points    = %3.6e s\n",time_step);
	printf("Delay from the trigger = %3.6e s\n",time_offset);
	printf("Timebase               = %s\n",bdt);
	printf("Calibre                = %s\n",cal);
#endif

	if(rec_type!=0) {
		fprintf(stderr,"ERROR : Only Single sweep is\n");
		fprintf(stderr,"currently supported\n");
		fprintf(stderr,"Current mode is %s\n",record_type[rec_type]);
		exit(1);
	}
 
	wave=(char*)malloc(nbpts*sizeof(char));

/* Get the waveform						*/
	sprintf(cmd,"%s:WF? DAT1",argv[1]);
	gpib_write(dso,cmd);
	gpib_read(dso,wave,nbpts);

/* Save data points						*/
	filename=(char *)malloc(strlen(argv[2])*sizeof(char));
	strcpy(filename,argv[2]);
	
	if((data=fopen(filename,"w"))==NULL) {
		fprintf(stderr,"Unable to create file %s\n",filename);
		exit(1);
	} else {	
		fprintf(data,"unset x\n");
		fprintf(data,"unset y%s\n",argv[1]);
		fprintf(data,"vector x(1)\n");
		fprintf(data,"vector y%s(1)\n",argv[1]);
		fprintf(data,"set bdt \"%s\"\n",bdt);
		fprintf(data,"set cal%s \"%s\"\n",argv[1],cal);
		fprintf(data,"set off%s %e\n",argv[1],offset);
		
		save_size=(long)((10.0*tdiv)/time_step);
		if(save_size>1000) {
			save_inc=(long)(floor(save_size/1000.0)+1.0);
		} else {
			save_inc=1;
		}
#ifdef DEBUG
		printf("Save step : %d\n",save_inc);
#endif
			
		
/* Save Y datas							*/
		fprintf(data,"set y%s(0) %d\n",argv[1],wave[22]);
		
		for(i=1;i<save_size;i=i+save_inc)
		{
			fprintf(data,"set y%s(++end) %d\n",argv[1],wave[22+i]);
		}

/* Save X datas							*/
		x_value=0;
		fprintf(data,"set x(0) 0\n");
		for(i=1;i<save_size;i=i+save_inc)
		{
			fprintf(data,"set x(++end) %d\n",x_value++);
		}
		fclose(data);
	}
	free(filename);
	free(wave);			
	exit(0);

}	
